#include <stdio.h>
#include <include/types.h>
#include <include/pflash.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <include/pmon.h>
static int
prhelp (const Cmd * cmd, int *lnp, int siz, int namemax, int optsmax)
{
	char prnbuf[LINESZ + 8];
	const Optdesc *p;
	int i;

	sprintf (prnbuf, "%*s  %-*s %s",
			 namemax, cmd->name, optsmax, cmd->opts, cmd->desc);
	if (more (prnbuf, lnp, siz))
	{
		return (1);
	}

	p = cmd->optdesc;
	if (p != 0)
	{
		for (i = 0; p[i].name; i++)
		{
			sprintf (prnbuf, "%*s  %15s    %s", namemax, "",
					 p[i].name, p[i].desc);
			if (more (prnbuf, lnp, siz))
			{
				return (1);
			}
		}
	}
	return (0);
}

static int cmd_help (int ac, char *av[])
{
	char prnbuf[LINESZ + 8];
	int i, j, namemax, optsmax, descmax, len;
	int ln, siz;
	int clistno;
	int expert;
	const Cmd *CmdTable;

	namemax = optsmax = descmax = 0;
	expert = getenv (EXPERT) != 0;

	clistno = 0;
	while ((CmdTable = CmdList[clistno++]) != 0)
	{
		for (i = 1; CmdTable[i].name != 0; i++)
		{
			len = strlen (CmdTable[i].name);
			if (len > namemax)
			{
				namemax = len;
			}
			len = strlen (CmdTable[i].opts);
			if (len > optsmax)
			{
				optsmax = len;
			}
			len = strlen (CmdTable[i].desc);
			if (len > descmax)
			{
				descmax = len;
			}
		}
	}

	siz = moresz;

	ln = siz;
	if (ac >= 2 && !strcmp (av[1], "*"))
	{							/* all commands */
		clistno = 0;
		while ((CmdTable = CmdList[clistno++]) != 0)
		{
			for (i = 1; CmdTable[i].name != 0; i++)
			{
				if ((CmdTable[i].flag & CMD_HIDE && !expert) ||
					CmdTable[i].flag & CMD_ALIAS)
				{
				}
				else if (prhelp (&CmdTable[i], &ln, siz, namemax, optsmax))
				{
					return (0);
				}
			}
		}
	}
	else if (ac >= 2)
	{							/* specific commands */
		for (j = 1; j < ac; j++)
		{
			clistno = 0;
			while ((CmdTable = CmdList[clistno++]) != 0)
			{
				for (i = 1; CmdTable[i].name != 0; i++)
				{
					if (!strcmp (CmdTable[i].name, av[j]))
					{
						break;
					}
				}
				if (CmdTable[i].name
					&& (!(CmdTable[i].flag & CMD_HIDE) || expert))
				{
					if (prhelp (&CmdTable[i], &ln, siz, namemax, optsmax))
					{
						return (0);
					}
					break;
				}
			}
			if (CmdTable == 0)
			{
				printf ("%s: not found\n", av[j]);
			}
		}
	}
	else
	{							/* general help only */
		const char *p = "";
		clistno = 0;
		i = 0;
		j = 0;
		while ((CmdTable = CmdList[clistno++]) != 0)
		{
			if (strcmp (CmdTable->name, p))
			{
				if (i % 2 != 0)
				{
					if (more (prnbuf, &ln, siz))
					{
						return (0);
					}
				}
				printf ("%70s\n", CmdTable->name);
				i = 0;
				j = 0;
				p = CmdTable->name;
			}
			CmdTable++;
			while (CmdTable->name != 0)
			{
				if (!(CmdTable->flag & CMD_ALIAS) &&
					(!(CmdTable->flag & CMD_HIDE) || expert))
				{
					j += sprintf (prnbuf + j, "%*s  %-*s", namemax,
								  CmdTable->name, descmax, CmdTable->desc);
					if (i % 2 != 0)
					{
						if (more (prnbuf, &ln, siz))
						{
							return (0);
						}
						j = 0;
					}
					else
					{
						j += sprintf (prnbuf + j, "   ");
					}
					i++;
				}
				CmdTable++;
			}
		}
		if (i % 2 != 0)
		{
			printf ("%s\n", prnbuf);
		}
	}
	return (0);
}
static const Cmd Cmds[] = {
	{"Shell"},
	{"h", "[*|cmd..]",
	 0,
	 "on-line help",
	 cmd_help, 1, 99, 0},
	{0, 0}
};
static void init_cmd __P ((void)) __attribute__ ((constructor));

static void init_cmd ()
{
	cmdlist_expand (Cmds, 0);
}
