#include <stdio.h>
#include <include/types.h>
#include <include/pflash.h>
#include <include/tgt/bonito.h>
#include <include/tgt/pflash_tgt.h>
#include <string.h>
#include <ctype.h>

/*
 *  gethex(vp,p,n) 
 *      convert n hex digits from p to binary, result in vp, 
 *      rtn 1 on success
 */
int gethex (int32_t * vp, char *p, int n)
{
	u_long v;
	int digit;

	for (v = 0; n > 0; n--)
	{
		if (*p == 0)
			return (0);
		if (*p >= '0' && *p <= '9')
			digit = *p - '0';
		else if (*p >= 'a' && *p <= 'f')
			digit = *p - 'a' + 10;
		else if (*p >= 'A' && *p <= 'F')
			digit = *p - 'A' + 10;
		else
			return (0);

		v <<= 4;
		v |= digit;
		p++;
	}
	*vp = v;
	return (1);
}

/** char *getword(dst,p) -- copies next word from p into dst, else rtns 0 */

char *getword (char *dst, const char *p)
{
	const char *a;

	if (!dst || !p)
		return (0);

	dst[0] = 0;
	while (isspace (*p))
		p++;
	if (*p == 0)
		return (0);
	a = p;
	while (!isspace (*p) && *p != 0)
		p++;
	strncpy (dst, a, p - a);
	dst[p - a] = 0;
	return ((char *) p);
}

/** int wordsz(p) -- return size of first word in p */

int wordsz (const char *p)
{
	int n;

	if (!p)
		return (0);

	while (isspace (*p))
		p++;
	for (n = 0; !*p && !isspace (*p); n++)
		p++;
	return (n);
}

/** char *strposn(p,q) returns a ptr to q in p, else 0 if not found */

char *strposn (const char *p, const char *q)
{
	const char *s, *t;

	if (!p || !q)
		return (0);

	if (!*q)
		return ((char *) (p + strlen (p)));
	for (; *p; p++)
	{
		if (*p == *q)
		{
			t = p;
			s = q;
			for (; *t; s++, t++)
			{
				if (*t != *s)
					break;
			}
			if (!*s)
				return ((char *) p);
		}
	}
	return (0);
}

#define BANCHOR		(0x80|'^')
#define EANCHOR		(0x80|'$')
#define MAXLN 256

/** int strpat(p,pat) return 1 if pat matches p, else 0; wildcards * and ? */


int strpat (const char *s1, const char *s2)
{
	char *p, *pat;
	char *t, tmp[MAXLN];
	char src1[MAXLN], src2[MAXLN];

	if (!s1 || !s2)
		return (0);

	p = src1;
	pat = src2;
	*p++ = BANCHOR;
	while (*s1)
		*p++ = *s1++;
	*p++ = EANCHOR;
	*p = 0;
	*pat++ = BANCHOR;
	while (*s2)
		*pat++ = *s2++;
	*pat++ = EANCHOR;
	*pat = 0;

	p = src1;
	pat = src2;
	for (; *p && *pat;)
	{
		if (*pat == '*')
		{
			pat++;
			for (t = pat; *t && *t != '*' && *t != '?'; t++);
			strncpy (tmp, pat, t - pat);
			tmp[t - pat] = '\0';
			pat = t;
			t = strposn (p, tmp);
			if (t == 0)
				return (0);
			p = t + strlen (tmp);
		}
		else if (*pat == '?' || *pat == *p)
		{
			pat++;
			p++;
		}
		else
			return (0);
	}
	if (!*p && !*pat)
		return (1);
	if (!*p && *pat == '*' && !*(pat + 1))
		return (1);
	return (0);
}

int strempty (const char *p)
{
	if (!p)
		return (1);
	for (; *p; p++)
		if (!isspace (*p))
			return (0);
	return (1);
}

#define MAX_AC 100

/*
 *  Take the string pointed by 's', break it up in words and
 *  make a pointer list in 'av'. Return number of 'args'.
 */
int argvize (char *av[], char *s)
{
	char **pav = av, c;
	int ac;

	for (ac = 0; ac < MAX_AC; ac++)
	{
		/* step over cntrls and spaces */
		while (*s && *s <= ' ')
			s++;

		if (!*s)
			break;

		c = *s;
		/* if it's a quote skip forward */
		if (c == '\'' || c == '"')
		{
			if (pav)
				*pav++ = ++s;
			while (*s && *s != c)
				s++;
			if (*s)
				*s++ = 0;
		}
		else
		{						/* find end of word */
			if (pav)
				*pav++ = s;
			while (' ' < *s)
				s++;
		}

		if (*s)
			*s++ = 0;
	}
	return (ac);
}
