#include <stdio.h>
#include <unistd.h>
#include <include/types.h>
#include <include/pflash.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include	<string.h>
#include <include/pmon.h>
#include <errno.h>
#include <getopt.h>

unsigned int mips_io_port_base, membase, bsucc;
static int quitnow = 0;

static enum {EEXIT_FAILURE = -1, EEXIT_SUCCESS} EXIT_STATUS;

static enum E_ARG_STATUS {NDEBUG, DEBUG, VERSION, HELP} ARG_STATUS;

static void setstatus(enum E_ARG_STATUS val)
{
	ARG_STATUS = ARG_STATUS < val ? val : ARG_STATUS;
}

static void usage()
{
	printf("\nUsage: pmon-install device\n");
	printf("Install pmon on device.\n\n");
	printf("  -h, --help\t\tprint this message and exit\n\n");
	printf("  -v, --version\t\tprint the version information and exit\n");
	printf("  device should be /dev/hdaxx or /dev/sdaxx.\n");
	printf("  xx is the number of your partition\n\n");
}

static void ver()
{
	printf("pmon-install (RAYS PMON-LOONGSON_CONFIG 0.1)\n");
}

int main (int argc, char *argv[])
{
	int fd;
	int iRet;
	FILE *fp;
	char str[201];
	char *pline;
	unsigned int flashaddr, flashsize;
	int flags;
	size_t bufsz;
	char col1[64], col2[64], col3[64];
	char *cmd[] = {"unset bootdev", "quit"};
	char *palpha = " abcdefghijklmnopqrstuvqxyz";
	int c;
	int option_index = 0;
	int para;

	static struct option long_options[] = 
	{
		{"help", 0, 0, 'h'},
		{"version", 0, 0, 'v'},
		{"debug", 0, 0, 'd'}
	};

	EXIT_STATUS = EEXIT_FAILURE;
	ARG_STATUS = NDEBUG;

	fd = open ("/dev/mem", O_RDWR);
	membase = mmap (0, 0x30000000, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0x10000000);
	if (membase == MAP_FAILED)
	{
		printf("membase map failed. error: [%s]\n", strerror(errno));
		return EXIT_STATUS;
	}
	close (fd);
	flashaddr = membase + 0x0fc00000;
	mips_io_port_base = membase + 0x0fd00000;
	envinit ();

	// set env success flag
	bsucc = 1;

	while (1)
	{
		c = getopt_long(argc, argv, "hvd", long_options, option_index);
		if (c == -1)
			break;

		switch (c)
		{
			case 'h':
				setstatus(HELP);
				break;
			case 'v':
				setstatus(VERSION);
				break;
			case 'd':
				setstatus(DEBUG);
				break;
			case '?':
				printf("parameter may be wrong\n");
				setstatus(HELP);
				break;
			default:
				break;
		}
	} // end of while (1)

	switch (ARG_STATUS)
	{
		case HELP:
			usage();
			goto out2;
		case VERSION:
			ver();
			goto out2;
		case DEBUG:
			while (!quitnow)
			{
				printf ("bioscfg>");
				fgets (str, 200, stdin);
				do_cmd (str);
			}
			goto out2;
		default:
			break;
	}

	// have no device parameter
	if (optind >= argc)
		goto b1;

	para = optind;

	if (strncmp(argv[para], "/dev/hd", 7) && strncmp(argv[para], "/dev/sd", 7) && strncmp(argv[para], "(wd", 3))
	{
		printf("Unknown device, pmon-install -h  or --help for help\n");
		goto out2;
	}

	if (!strncmp(argv[para], "/dev/hda", 8) || 
		!strncmp(argv[para], "/dev/sda", 8) || 
		!strncmp(argv[para], "/dev/hdb", 8) || 
		!strncmp(argv[para], "/dev/sdb", 8))
	{
		iRet = 0;
		pline = argv[para] + 8;
		while (*pline)
		{
			if ((*pline >= '0') && (*pline <= '9'))
			{
				iRet = iRet * 10 + *pline - '0';
			}
			else
			{
				printf("Partition Number is wrong\n");
				EXIT_STATUS = EEXIT_FAILURE ;
				goto out2;
			}
			pline++;
		}
		if (iRet) 
			iRet -= 1;

		memset(str, 0x0, sizeof(str));
		pline = argv[para] + 7;
		switch (*pline)
		{
			case 'a':
				sprintf(str, "set bootdev \"(wd0,%d)\"", iRet);
				break;
			case 'b':
				sprintf(str, "set bootdev \"(wd1,%d)\"", iRet);
				break;
			default:
				break;
		}

		goto b2;
	}
	else
	{
		memset(str, 0x0, sizeof(str));
		if (strlen(argv[para]) > 100)
		{
			EXIT_STATUS = EEXIT_FAILURE ;
			goto out2;
		}
		sprintf(str, "set bootdev \"%s\"", argv[para]);
		goto b2;
	}

b1:
	// do something
	memset(str, 0x0, sizeof(str));
	strcpy(str, "set bootdev");
	do_cmd(str);
	goto out2;

b2:
	do_cmd(str);

	if (!bsucc)
	{
		do_cmd(cmd[0]);
		do_cmd(cmd[1]);
	}

	if (argc > 1)
	{
		memset(str, 0x0, sizeof(str));
		strcpy(str, "set bootdev");
		do_cmd(str);

		printf("bootdev set successful.\n");
	}
	else 
	{
		printf("You can use pmon-install to set bootdev, pmon-install -h or --help for help\n");
	}

out2:
	munmap (membase, 0x10000000);
	return EXIT_STATUS;
}
static cmd_quit (int ac, char *av[])
{
	quitnow = 1;
	return 0;
}

static const Cmd Cmds[] = {
	{"Main"},
	{"quit", "",
	 0,
	 "quit",
	 cmd_quit, 1, 1, CMD_REPEAT},
	{0, 0}
};

static void init_cmd __P ((void)) __attribute__ ((constructor));

static void init_cmd ()
{
	cmdlist_expand (Cmds, 1);
}
