#include <stdio.h>
#include <include/types.h>
#include <include/pflash.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <include/pmon.h>
#include <unistd.h>

extern unsigned int mips_io_port_base, membase;
static int cmd_program (int ac, char *av[])
{
	unsigned int flashaddr, flashsize, addr, size;
	struct stat state;
	int ret;
	char *buf;
	FILE *fp;
	flashaddr = strtoul (av[1], 0, 0);
	flashaddr += membase - 0x10000000;
	ret = stat (av[2], &state);
	size = state.st_size;
	if (ret < 0)
		return -1;

	tgt_flashinfo (flashaddr, &flashsize);
	if (flashsize == 0)
	{
		printf ("No FLASH at given address\n");
		return -2;
	}
	if (flashsize < size)
	{
		printf ("file size %d to big for FLASH size %d\n", size, flashsize);
		return -2;
	}
	buf = malloc (size);
	if (!buf)
	{
		printf ("malloc(%d) failed %d\n", size, buf);
		return -3;
	}
	fp = fopen (av[2], "rb");
	if (!fp)
	{
		printf ("open file failed\n");
		return -4;
	}
	fread (buf, size, 1, fp);
	fclose (fp);
	tgt_flashprogram ((void *) flashaddr,	/* address */
					  size,		/* size */
					  (void *) buf,	/* load */
					  0);
	return 0;
}

static int cmd_save (int ac, char *av[])
{
	unsigned int flashaddr, flashsize, addr, size;
	struct stat state;
	int ret;
	char *buf;
	FILE *fp;
	flashaddr = strtoul (av[1], 0, 0);
	flashaddr += membase - 0x10000000;
	size = strtoul (av[3], 0, 0);

	tgt_flashinfo (flashaddr, &flashsize);
	if (flashsize == 0)
	{
		printf ("No FLASH at given address\n");
		return -2;
	}
	if (flashsize < size)
	{
		printf ("file size %d to big,set to FLASH size %d\n", size,
				flashsize);
		size = flashsize;
	}
	fp = fopen (av[2], "wb");
	if (!fp)
	{
		printf ("open file failed\n");
		return -4;
	}
	fwrite (flashaddr, size, 1, fp);
	fclose (fp);
	return 0;
}

static const Cmd Cmds[] = {
	{"Main"},
	{"program", "addr file",
	 0,
	 "progran a file into flash at at address",
	 cmd_program, 3, 3, CMD_REPEAT},
	{"save", "addr file size",
	 0,
	 "save data from flash at at address",
	 cmd_save, 4, 4, CMD_REPEAT},
	{0, 0}
};

static void init_cmd __P ((void)) __attribute__ ((constructor));

static void init_cmd ()
{
	cmdlist_expand (Cmds, 1);
}
